@@ -12,6 +12,8 @@
 
 #include <chrono>
 
+#include <QGuiApplication>
+
 #include <KIO/TransferJob>
 #include <KLocalizedString>
 
@@ -38,7 +40,6 @@ static const int s_maxReportSize = 65535;
 
 ReportInterface::ReportInterface(QObject *parent)
     : QObject(parent)
-    , m_duplicate(0)
     , m_sentryPostbox(DrKonqi::crashedApplication()->fakeExecutableBaseName(), std::make_shared<SentryNetworkConnection>())
 {
     m_bugzillaManager = new BugzillaManager(KDE_BUGZILLA_URL, this);
@@ -114,16 +115,6 @@ void ReportInterface::setBacktrace(const QString &backtrace)
     Q_EMIT backtraceChanged();
 }
 
-QStringList ReportInterface::firstBacktraceFunctions() const
-{
-    return m_firstBacktraceFunctions;
-}
-
-void ReportInterface::setFirstBacktraceFunctions(const QStringList &functions)
-{
-    m_firstBacktraceFunctions = functions;
-}
-
 QString ReportInterface::title() const
 {
     return m_reportTitle;
@@ -141,11 +132,6 @@ void ReportInterface::setDetailText(const QString &text)
     Q_EMIT detailTextChanged();
 }
 
-void ReportInterface::setPossibleDuplicates(const QStringList &list)
-{
-    m_possibleDuplicates = list;
-}
-
 QString ReportInterface::generateReportFullText(DrKonqiStamp stamp, Backtrace inlineBacktrace) const
 {
     // Note: no translations should be done in this function's strings
@@ -237,29 +223,6 @@ QString ReportInterface::generateReportFullText(DrKonqiStamp stamp, Backtrace in
         report.append(QStringLiteral("A useful backtrace could not be generated\n"));
     }
 
-    // Possible duplicates (selected by the user)
-    if (!m_possibleDuplicates.isEmpty()) {
-        report.append(QLatin1Char('\n'));
-        QString duplicatesString;
-        for (const QString &dupe : std::as_const(m_possibleDuplicates)) {
-            duplicatesString += QLatin1String("bug ") + dupe + QLatin1String(", ");
-        }
-        duplicatesString = duplicatesString.left(duplicatesString.length() - 2) + QLatin1Char('.');
-        report.append(QStringLiteral("The reporter indicates this bug may be a duplicate of or related to %1\n").arg(duplicatesString));
-    }
-
-    // Several possible duplicates (by bugzilla query)
-    if (!m_allPossibleDuplicatesByQuery.isEmpty()) {
-        report.append(QLatin1Char('\n'));
-        QString duplicatesString;
-        int count = m_allPossibleDuplicatesByQuery.count();
-        for (int i = 0; i < count && i < 5; i++) {
-            duplicatesString += QLatin1String("bug ") + m_allPossibleDuplicatesByQuery.at(i) + QLatin1String(", ");
-        }
-        duplicatesString = duplicatesString.left(duplicatesString.length() - 2) + QLatin1Char('.');
-        report.append(QStringLiteral("Possible duplicates by query: %1\n").arg(duplicatesString));
-    }
-
     switch (stamp) {
     case DrKonqiStamp::Include: {
         report.append(QLatin1String("\nReported using DrKonqi"));
@@ -363,36 +326,75 @@ void ReportInterface::prepareEventPayload()
     // replace the timestamp with the real timestamp (possibly originating in journald)
     hash.insert(u"timestamp"_s, DrKonqi::crashedApplication()->datetime().toUTC().toString(Qt::ISODateWithMs));
 
-    QList<QVariant> breadcrumbs;
-    const auto logs = DrKonqi::crashedApplication()->m_logs;
-    for (const auto &logEntry : logs) {
-        QVariantHash breadcrumb;
+    {
+        QList<QVariant> breadcrumbs;
+        const auto logs = DrKonqi::crashedApplication()->m_logs;
+        for (const auto &logEntry : logs) {
+            QVariantHash breadcrumb;
 
-        breadcrumb.insert(u"type"_s, u"debug"_s);
-        breadcrumb.insert(u"category"_s, logEntry.value("QT_CATEGORY"_ba, "default"_ba));
-        breadcrumb.insert(u"message"_s, logEntry.value("MESSAGE"_ba));
-        breadcrumb.insert(u"level"_s, journalPriorityToSentryLevel(logEntry.value("PRIORITY"_ba)));
-        auto ok = false;
-        auto realtime = logEntry.value("_SOURCE_REALTIME_TIMESTAMP"_ba).toULongLong(&ok);
-        if (ok && realtime > 0) {
-            std::chrono::microseconds timestamp(realtime);
-            std::chrono::duration<double> timestampDouble(timestamp);
-            breadcrumb.insert(u"timestamp"_s, QJsonValue(timestampDouble.count()));
+            breadcrumb.insert(u"type"_s, u"debug"_s);
+            breadcrumb.insert(u"category"_s, logEntry.value("QT_CATEGORY"_ba, "default"_ba));
+            breadcrumb.insert(u"message"_s, logEntry.value("MESSAGE"_ba));
+            breadcrumb.insert(u"level"_s, journalPriorityToSentryLevel(logEntry.value("PRIORITY"_ba)));
+            auto ok = false;
+            auto realtime = logEntry.value("_SOURCE_REALTIME_TIMESTAMP"_ba).toULongLong(&ok);
+            if (ok && realtime > 0) {
+                std::chrono::microseconds timestamp(realtime);
+                std::chrono::duration<double> timestampDouble(timestamp);
+                breadcrumb.insert(u"timestamp"_s, QJsonValue(timestampDouble.count()));
+            }
+
+            breadcrumbs.push_back(breadcrumb);
         }
-
-        breadcrumbs.push_back(breadcrumb);
+        std::reverse(breadcrumbs.begin(), breadcrumbs.end());
+        hash.insert(u"breadcrumbs"_s, breadcrumbs);
     }
-    std::reverse(breadcrumbs.begin(), breadcrumbs.end());
-    hash.insert(u"breadcrumbs"_s, breadcrumbs);
 
-    m_sentryPostbox.addEventPayload(SentryEvent(QJsonDocument::fromVariant(hash).toJson()));
+    {
+        constexpr auto CONTEXTS_KEY = "contexts"_L1;
+        auto context = hash.take(CONTEXTS_KEY).toHash();
+        context.insert(u"gpu"_s,
+                       QVariantHash{
+                           {u"name"_s, DrKonqi::instance()->m_glRenderer}, //
+                           {u"version"_s, QGuiApplication::platformName()},
+                       });
+        hash.insert(CONTEXTS_KEY, context);
+    }
+
+    if (!DrKonqi::instance()->m_exceptionName.isEmpty()) {
+        constexpr auto EXCEPTION_KEY = "exception"_L1;
+        constexpr auto VALUES_KEY = "values"_L1;
+        auto exception = hash.take(EXCEPTION_KEY).toHash();
+        auto values = exception.take(VALUES_KEY).toJsonArray();
+        // Preprend since the exception is likely the root of the problem stack.
+        values.prepend(QJsonObject({
+            {u"type"_s, DrKonqi::instance()->m_exceptionName},
+            {u"value"_s, DrKonqi::instance()->m_exceptionWhat},
+            {u"mechanism"_s,
+             QJsonObject{
+                 {u"handled"_s, false},
+                 {u"synthetic"_s, false},
+                 {u"type"_s, u"drkonqi"_s},
+             }},
+        }));
+        exception.insert(VALUES_KEY, values);
+        hash.insert(EXCEPTION_KEY, exception);
+    }
+
+    m_sentryPostbox.addEventPayload(QJsonDocument::fromVariant(hash));
     maybePickUpPostbox();
 }
 
 void ReportInterface::prepareCrashEvent()
 {
-    if (DrKonqi::debuggerManager()->backtraceGenerator()->state() == BacktraceGenerator::Loaded) {
+    switch (DrKonqi::debuggerManager()->backtraceGenerator()->state()) {
+    case BacktraceGenerator::Loaded:
         return prepareEventPayload();
+    case BacktraceGenerator::Loading:
+    case BacktraceGenerator::NotLoaded:
+    case BacktraceGenerator::Failed:
+    case BacktraceGenerator::FailedToStart:
+        break;
     }
     static bool connected = false;
     if (!connected) {
@@ -413,55 +415,52 @@ void ReportInterface::prepareCrashComment()
     maybePickUpPostbox();
 }
 
+void ReportInterface::createCrashMessage(const QString &message)
+{
+    m_sentryPostbox.addUserFeedback(message);
+    maybePickUpPostbox();
+}
+
 void ReportInterface::sendBugReport()
 {
     prepareCrashEvent();
 
-    if (m_attachToBugNumber > 0) {
-        // We are going to attach the report to an existent one
-        connect(m_bugzillaManager, &BugzillaManager::addMeToCCFinished, this, &ReportInterface::attachBacktraceWithReport);
-        connect(m_bugzillaManager, &BugzillaManager::addMeToCCError, this, &ReportInterface::sendReportError);
-        // First add the user to the CC list, then attach
-        m_bugzillaManager->addMeToCC(m_attachToBugNumber);
-    } else {
-        // Creating a new bug report
-        bool attach = false;
-        Bugzilla::NewBug report = newBugReportTemplate();
-        report.description = generateReportFullText(ReportInterface::DrKonqiStamp::Include, ReportInterface::Backtrace::Complete);
+    bool attach = false;
+    Bugzilla::NewBug report = newBugReportTemplate();
+    report.description = generateReportFullText(ReportInterface::DrKonqiStamp::Include, ReportInterface::Backtrace::Complete);
 
-        // If the report is too long try to reduce it, try to not include the
-        // backtrace and eventually give up.
-        // Bugzilla has a hard-limit on the server side, if we cannot strip the
-        // report down enough the submission will simply not work.
-        // Exhausting the cap with just user input is nigh impossible, so we'll
-        // forego handling of the report being too long even without without
-        // backtrace.
-        // https://bugs.kde.org/show_bug.cgi?id=248807
-        if (report.description.size() >= s_maxReportSize) {
-            report.description = generateReportFullText(ReportInterface::DrKonqiStamp::Include, ReportInterface::Backtrace::Reduced);
-            attach = true;
-        }
-        if (report.description.size() >= s_maxReportSize) {
-            report.description = generateReportFullText(ReportInterface::DrKonqiStamp::Include, ReportInterface::Backtrace::Exclude);
-            attach = true;
-        }
-        Q_ASSERT(!report.description.isEmpty());
-
-        connect(m_bugzillaManager, &BugzillaManager::sendReportErrorInvalidValues, this, &ReportInterface::sendUsingDefaultProduct);
-        connect(m_bugzillaManager, &BugzillaManager::reportSent, this, [this, attach](int bugId) {
-            if (attach) {
-                m_attachToBugNumber = bugId;
-                attachBacktrace(QStringLiteral("DrKonqi auto-attaching complete backtrace."));
-            } else {
-                m_sentReport = bugId;
-                prepareCrashComment();
-                m_sentryPostbox.deliver();
-                maybeDone();
-            }
-        });
-        connect(m_bugzillaManager, &BugzillaManager::sendReportError, this, &ReportInterface::sendReportError);
-        m_bugzillaManager->sendReport(report);
+    // If the report is too long try to reduce it, try to not include the
+    // backtrace and eventually give up.
+    // Bugzilla has a hard-limit on the server side, if we cannot strip the
+    // report down enough the submission will simply not work.
+    // Exhausting the cap with just user input is nigh impossible, so we'll
+    // forego handling of the report being too long even without without
+    // backtrace.
+    // https://bugs.kde.org/show_bug.cgi?id=248807
+    if (report.description.size() >= s_maxReportSize) {
+        report.description = generateReportFullText(ReportInterface::DrKonqiStamp::Include, ReportInterface::Backtrace::Reduced);
+        attach = true;
     }
+    if (report.description.size() >= s_maxReportSize) {
+        report.description = generateReportFullText(ReportInterface::DrKonqiStamp::Include, ReportInterface::Backtrace::Exclude);
+        attach = true;
+    }
+    Q_ASSERT(!report.description.isEmpty());
+
+    connect(m_bugzillaManager, &BugzillaManager::sendReportErrorInvalidValues, this, &ReportInterface::sendUsingDefaultProduct);
+    connect(m_bugzillaManager, &BugzillaManager::reportSent, this, [this, attach](int bugId) {
+        if (attach) {
+            m_attachToBugNumber = bugId;
+            attachBacktrace(QStringLiteral("DrKonqi auto-attaching complete backtrace."));
+        } else {
+            m_sentReport = bugId;
+            prepareCrashComment();
+            m_sentryPostbox.deliver();
+            maybeDone();
+        }
+    });
+    connect(m_bugzillaManager, &BugzillaManager::sendReportError, this, &ReportInterface::sendReportError);
+    m_bugzillaManager->sendReport(report);
 }
 
 void ReportInterface::sendUsingDefaultProduct() const
@@ -565,22 +564,6 @@ uint ReportInterface::attachToBugNumber() const
     return m_attachToBugNumber;
 }
 
-void ReportInterface::setDuplicateId(uint duplicate)
-{
-    m_duplicate = duplicate;
-    Q_EMIT duplicateIdChanged();
-}
-
-uint ReportInterface::duplicateId() const
-{
-    return m_duplicate;
-}
-
-void ReportInterface::setPossibleDuplicatesByQuery(const QStringList &list)
-{
-    m_allPossibleDuplicatesByQuery = list;
-}
-
 BugzillaManager *ReportInterface::bugzillaManager() const
 {
     return m_bugzillaManager;
@@ -606,14 +589,15 @@ ReportInterface *ReportInterface::self()
 
 bool ReportInterface::hasCrashEventSent() const
 {
-    return !isCrashEventSendingEnabled() || m_sentryPostbox.hasDelivered();
+    return !isCrashEventSendingEnabled() || m_sentryPostbox.hasDelivered() || m_skipSentry;
 }
 
 bool ReportInterface::isCrashEventSendingEnabled() const
 {
     qCDebug(DRKONQI_LOG) << "sentry:" << Settings::self()->sentry() //
                          << "forceSentry:" << m_forceSentry //
-                         << "hasDeletedFiles:" << DrKonqi::crashedApplication()->hasDeletedFiles();
+                         << "hasDeletedFiles:" << DrKonqi::crashedApplication()->hasDeletedFiles() //
+                         << "skipSentry:" << m_skipSentry;
     const auto enabled = Settings::self()->sentry() || m_forceSentry;
     return enabled && !DrKonqi::crashedApplication()->hasDeletedFiles();
 }
@@ -651,4 +635,9 @@ void ReportInterface::trySentry()
     }
 }
 
+QString ReportInterface::sentryEventId() const
+{
+    return m_sentryPostbox.eventId();
+}
+
 #include "moc_reportinterface.cpp"
